'use strict';
var utils = require('./utils');
let Router = require('koa-router');
(function (HttpMethod) {
    HttpMethod[HttpMethod["HEAD"] = 0] = "HEAD";
    HttpMethod[HttpMethod["OPTIONS"] = 1] = "OPTIONS";
    HttpMethod[HttpMethod["GET"] = 2] = "GET";
    HttpMethod[HttpMethod["PUT"] = 3] = "PUT";
    HttpMethod[HttpMethod["PATCH"] = 4] = "PATCH";
    HttpMethod[HttpMethod["POST"] = 5] = "POST";
    HttpMethod[HttpMethod["DELETE"] = 6] = "DELETE";
})(exports.HttpMethod || (exports.HttpMethod = {}));
var HttpMethod = exports.HttpMethod;
function route(path, method, ...middleware) {
    return (target, key, descriptor) => {
        utils.assert(!!path, '@route should have at least "path" argument');
        if (!target.prototype.router) {
            target.prototype.router = new Router();
        }
        if (typeof target === 'function' && key === undefined && descriptor === undefined) {
            utils.assert(!method, '@route on Type should not have "method"');
            target.prototype.router.prefix(path);
            if (middleware.length > 0) {
                target.prototype.router.use(...middleware);
            }
            return;
        }
        utils.assert(!!method, '@route on method should have "method" as second argument');
        switch (method) {
            case HttpMethod.HEAD:
                target.prototype.router.head(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.OPTIONS:
                target.prototype.router.options(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.GET:
                target.prototype.router.get(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.PUT:
                target.prototype.router.put(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.PATCH:
                target.prototype.router.patch(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.POST:
                target.prototype.router.post(path, ...middleware, descriptor.value);
                break;
            case HttpMethod.DELETE:
                target.prototype.router.delete(path, ...middleware, descriptor.value);
                break;
            default:
                throw new Error('@route decorator "method" is not valid');
        }
    };
}
exports.route = route;
